{- | Logica proposicional

Este modulo da el tipo Operador para la logica proposicional, asi como las transformaciones
basicas que seran utiles para todos los ejercicios (por ejemplo, la doble negacion).

Ademas, se incluyen aqui los ejercicios tipo sobre logica proposicional que Principia lleve. El
usuario tiene libertad para definir sus ejercicios tipo aqui o en un modulo a parte que importe
este.
-}
module Proposicional(
	Operador(..),
	comunA1,
	solucionesA1,
	module Monad,
	module Expresion,
	module Traza
	) where
import Monad
import Expresion
import Traza

-------------------------
--EXPRESIONES DE EJEMPLO
-------------------------
test1 = O OR [ V "a", O NOT [O AND [V "b",O NOT [V "c"]]] ]

-------------------------
--COMUN
-------------------------
data Operador = 
	NOT			-- ^ negacion
	|AND			-- ^ conjuncion
	|OR			-- ^ disyuncion inclusiva
	|SI			-- ^ condicional
	|SII			-- ^ bicondicional
	deriving (Eq,Show)

{- | Simplificaciones disponibles para todos los tipos de ejercicio. 

A1 se refiere a "alfa1", la version de Principia en que se ha introducido esta lista. La lista
se mantendra en posteriores versiones de principia, y simplemente se podran annadir otras,
como una hipotetica "comunA5" o "comunB2" o "comunV1"; de esta forma se facilita que cualquier
tipo de ejercicio que un usuario haya podido definir, siga funcionando en versiones posteriores.

-}
comunA1 :: [Transformacion Bool Operador]
comunA1 = [
		(O NOT [O NOT [V "p"]], 		--doble negacion
		V "p")
	]

solucionesA1 :: [Transformacion Bool Operador] -> Expresion Bool Operador -> [String]
solucionesA1 transformaciones e =	
	let resultados = simplificar transformaciones e in
	let inicio = show e ++" (dato)\n" in
	[inicio ++ traza r ++ fin| r<-resultados, let fin = (show.contenido) r ++ "\n"] 

-- Modificar en versiones posteriores, y revisar que los ejercicios siguen funcionando bien.
-- NO EXPORTAR:
comun = comunA1
soluciones = solucionesA1

-------------------------
--Forma clausulada
-------------------------
-- |Transformaciones para conseguir una forma clausulada.
--
--En todos los ejercicios se seguira el patron t++tipoEjercicio a la hora de llamar a esta lista.
tClausulada :: [Transformacion Bool Operador]
tClausulada = comun ++
	[	
		(O SII [V "p",V "q"] ,			--definicion de bicondicional 
		O OR [O SI [V "p",V "q"],O SI [V "q",V "p"]]),
		
		(O SI [V "p",V "q"] , 
		O OR [O NOT [V "p"],V "q"]), 		--definicion de condicional

		(O NOT [O AND [V "p",V "q"]] , 
		O OR [O NOT [V "p"],O NOT [V "q"]]), 	--ley de Morgan I
		
		(O NOT [O OR [V "p",V "q"]] , 
		O AND [O NOT [V "p"],O NOT [V "q"]]), 	--ley de Morgan II
		
		(O OR [V "p",O AND [V "q",V "r"]] , 
		O AND [O OR [V "p",V "q"],O OR [V "p",V "r"]]) --prop distributiva
	]

-- |Soluciona un ejercicio tipo 'transformar en forma clausulada'.
--
-- En todos los ejercicios se seguira el patron s++tipoEjercicio a la hora de llamar a esta
-- funcion.
sClausulada :: Expresion Bool Operador -> IO()
sClausulada = putStr . head . soluciones tClausulada
